import os, json


def get_request(local_file="request.json"):
    try:
        with open(os.getenv("REQUEST_FILE", local_file)) as f:
            request = json.loads(f.read())
            body = request["body"]
            query = request["query"]
            headers = request["headers"]

        try:
            if headers["Content-Type"] == "application/json":
                return json.loads(body), query, headers
            else:
                return body, query, headers
        except Exception:
            return body, query, headers

    except Exception:
        return None, None, None


def send_response(body="", status_code=200, headers={}, local_file="response.json"):
    response = {"status_code": status_code, "body": body, "headers": headers}
    path = os.getenv("RESPONSE_FILE", local_file)
    content = json.dumps(response)
    with open(path, "w") as f:
        f.write(content)


def send_json(data={}, status_code=200, headers={}, local_file="response.json"):
    send_response(
        body=json.dumps(data, allow_nan=False),
        headers={**headers, "content-type": "application/json"},
        status_code=status_code,
        local_file=local_file,
    )



import json
from typing import Dict


def deserialize(string: str) -> Dict:
    return json.loads(string)


def serialize(obj: Dict) -> bytes:
    return bytes(json.dumps(obj), "UTF-8")


def get_headers(api_key: str) -> Dict:
    return {"Api-Authorization": f"Bearer {api_key}"}



from io import IOBase
from typing import Optional, Union, List
from ..apis import upload_file
from ..widget_base import Input
from ..response_types import FileResponse


class ImageInput(Input):
    type = "image-input"

    def __init__(self, key: str, label: str, **kwargs):
        super().__init__(key)
        self.label = label
        self.initial_value = kwargs.get("initial_value", "")
        self.required = kwargs.get("required", True)
        self.hint = kwargs.get("hint", None)
        self.columns = kwargs.get("columns", 1)
        self.multiple = kwargs.get("multiple", False)
        self.full_width = kwargs.get("full_width", False)
        self.disabled = kwargs.get("disabled", False)
        self.max_file_size = kwargs.get("max_file_size", None)

    def json(self, **kwargs):
        return {
            "type": self.type,
            "key": self.key,
            "hint": self.hint,
            "label": self.label,
            "initialValue": ImageInput.__revert_value(self.initial_value),
            "columns": self.columns,
            "required": self.required,
            "multiple": self.multiple,
            "fullWidth": self.full_width,
            "disabled": self.disabled,
            "maxFileSize": self.max_file_size,
        }

    @staticmethod
    def __convert_value(value: Union[FileResponse, str, IOBase]) -> str:
        if isinstance(value, FileResponse):
            return value.url
        if isinstance(value, str):
            return value
        if isinstance(value, IOBase):
            return upload_file(value)
        return ""

    @staticmethod
    def __revert_value(
        value: Optional[
            Union[
                FileResponse, List[FileResponse], str, List[str], IOBase, List[IOBase]
            ]
        ]
    ) -> Optional[str]:
        if not value:
            return None
        if isinstance(value, list):
            return [ImageInput.__convert_value(item) for item in value]
        return ImageInput.__convert_value(value)

    @staticmethod
    def __convert_answer(answer) -> Optional[FileResponse]:
        if not answer:
            return None
        if isinstance(answer, list):
            return [FileResponse(item) for item in answer]
        return FileResponse(answer)

    def convert_answer(self, answer) -> Optional[FileResponse]:
        """
        Returns:
            FileResponse or FileResponse[]: A dict containing the image file uploaded by the user ({"file": file, "url": str, "content": bytes}) or a list of images in case of multiple flag set as True
        """
        return self.__convert_answer(answer)



# This is an interface only file
# Actual functions are defined on runtime


class AuthResponse:
    email: str


def get_user() -> AuthResponse:
    pass


def redirect(url: str, params={}):
    pass


def get_query_params() -> dict:
    pass



from .ProgressOutput import ProgressOutput
from .TextOutput import TextOutput
from .CustomInput import CustomInput
from .MultipleChoiceInput import MultipleChoiceInput
from .MarkdownOutput import MarkdownOutput
from .CodeInput import CodeInput
from .KanbanBoardInput import KanbanBoardInput
from .ImageOutput import ImageOutput
from .ClickInput import ClickInput
from .PhoneInput import PhoneInput
from .LinkOutput import LinkOutput
from .AnswerSheetInput import AnswerSheetInput
from .TextareaInput import TextareaInput
from .DateInput import DateInput
from .DropdownInput import DropdownInput
from .PlotlyOutput import PlotlyOutput
from .TagInput import TagInput
from .CurrencyInput import CurrencyInput
from .CheckboxInput import CheckboxInput
from .EmailInput import EmailInput
from .ChecklistInput import ChecklistInput
from .PandasRowSelectionInput import PandasRowSelectionInput
from .VideoInput import VideoInput
from .CardsInput import CardsInput
from .CnpjInput import CnpjInput
from .LatexOutput import LatexOutput
from .RatingInput import RatingInput
from .FileInput import FileInput
from .HtmlOutput import HtmlOutput
from .ToggleInput import ToggleInput
from .ListInput import ListInput
from .NumberInput import NumberInput
from .IframeOutput import IframeOutput
from .ImageInput import ImageInput
from .PandasOutput import PandasOutput
from .CpfInput import CpfInput
from .TextInput import TextInput
from .PasswordInput import PasswordInput
from .NumberSliderInput import NumberSliderInput
from .FileOutput import FileOutput
from .NpsInput import NpsInput
from .TimeInput import TimeInput



from ..widget_base import Input


class TextInput(Input):
    type = "text-input"

    def __init__(self, key: str, label: str, **kwargs):
        super().__init__(key)
        self.label = label
        self.initial_value = kwargs.get("initial_value", "")
        self.placeholder = kwargs.get("placeholder", "Your answer here")
        self.required = kwargs.get("required", True)
        self.hint = kwargs.get("hint", None)
        self.columns = kwargs.get("columns", 1)
        self.full_width = kwargs.get("full_width", False)
        self.mask = kwargs.get("mask", None)
        self.disabled = kwargs.get("disabled", False)

    def json(self, **kwargs):
        return {
            "type": self.type,
            "key": self.key,
            "label": self.label,
            "initialValue": self.initial_value,
            "placeholder": self.placeholder,
            "required": self.required,
            "hint": self.hint,
            "columns": self.columns,
            "fullWidth": self.full_width,
            "mask": self.mask,
            "disabled": self.disabled,
        }

    def convert_answer(self, answer: str) -> str:
        """
        Returns:
            str: The value entered by the user
        """
        return answer
